# Registry of staff identities
Allows you to find staff identities by id using *Soar::Registry::Staff::Identity::Id* or email *Soar::Registry::Staff::Identity::Email*

## Quickstart

### Required configuration
See config/config.yml
```yaml
provider:
  class: 'Soar::Registry::Staff::Directory::DynamoDb::Identity'
  config: 
    table: 'identities'
    region: 'us-west-2'
    endpoint: 'http://localhost:8000'
  credentials:
    username: 'username'
    password: 'secret'
```

### Create an instance of the chosen staff identity registry

#### Find by id
```ruby
require 'soar/registry/staff/identity/id'
@id_idr = Soar::Registry::Staff::Identity::Id.new(configuration)
```
#### Find by email
```ruby
require 'soar/registry/staff/identity/email'
@email_idr = Soar::Registry::Staff::Identity::Email.new(configuration)
```

### Getting a list of identifiers
```ruby
> identifiers = @id_idr.get_identifiers("identity-820d5660-2204-4f7d-8c04-746313439b81") 
> identifiers = @email_idr.get_identifiers("admin@hetzner.co.za") 
> puts identifiers.inspect
["admin@hetzner.co.za", "identity-820d5660-2204-4f7d-8c04-746313439b81"]
```

### Getting a list of roles
```ruby
> roles = @id_idr.get_roles("identity-820d5660-2204-4f7d-8c04-746313439b81")
> roles = @email_idr.get_roles("admin@hetzner.co.za")
> puts roles.inspect
["staff", "configuration_publisher", "configuration_consumer"]
```

### Getting a hash of attributes for a role 
```ruby
> role = 'staff'
> attributes = @id_idr.get_attributes("identity-820d5660-2204-4f7d-8c04-746313439b81", role)
> attributes = @email_idr.get_attributes("admin@hetzner.co.za", role)
> puts attributes.inspect
{
    "staff": {
        "department": "technical"
    }
}

```

### Getting a hash of all attributes
```ruby
> attributes = @id_idr.get_attributes("identity-820d5660-2204-4f7d-8c04-746313439b81")
> attributes = @email_idr.get_attributes("admin@hetzner.co.za")
> puts attributes.inspect
{
    "identity_id" => "identity-820d5660-2204-4f7d-8c04-746313439b81",
    "entity_id"=> "entity-bad85eb9-0713-4da7-8d36-07a8e4b00eab",
    "email"=> "admin@hetzner.co.za",
    "roles"=> {
        "staff"=> {},
        "configuration_publisher"=> {
            "configuration_identifiers"=> ["*"]
        },
        "configuration_consumer"=> {
            "configuration_identifiers"=> ["*"]
        }
    },
    "address"=> {
        "detail"=> "Belvedere Office Park, Unit F",
        "street"=> "Bella Rosa Street",
        "suburb"=> "Tygervalley",
        "city"=> "Durbanville",
        "postal"=> "7550"
    }
}
```


## Quicktest

### Local Stub 
```bash
TEST_ORCHESTRATION_PROVIDER=Stub rspec
```

### DynamoDb
```
$ docker-compose --file docker-compose.test.yml up
$ CONFIG_FILE=config.test.yml TEST_ORCHESTRATION_PROVIDER=DynamoDb rspec
```

### CI
*NB:* container_name in docker-compose.ci.yml corresponds with name parameter of docker ps command in below bash script.
```bash
USER_NAME=$USER USER_ID=$(id -u) docker-compose --file docker-compose.ci.yml  up --abort-on-container-exit --remove-orphans --build --force-recreate
EXIT_CODE=$(docker ps -a -f "name=soar-registry-staff" -q | xargs docker inspect -f "{{ .State.ExitCode }}");
exit $EXIT_CODE;
```


## Useful commands:
```
$ aws dynamodb list-tables --endpoint-url http://localhost:8000
$ aws dynamodb delete-table --table-name identities --endpoint-url http://localhost:8000
```

## Resources
* [DynamoDBLocal](http://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DynamoDBLocal.html)
* [Multiple AWS Credentials](https://blogs.aws.amazon.com/security/post/Tx3D6U6WSFGOK2H/A-New-and-Standardized-Way-to-Manage-Credentials-in-the-AWS-SDKs)
* [AWS SDK for Ruby](http://docs.aws.amazon.com/amazondynamodb/latest/gettingstartedguide/GettingStarted.Ruby.html)

## References
* [soar idm](https://github.hetzner.co.za/hetznerZA/soar_idm/blob/master/lib/soar_idm/soar_idm.rb)
* [Domain analysis](https://docs.google.com/a/hetzner.co.za/drawings/d/1vGdzjKPD3gzn1e0bsC4liFCyxY31Qjjxe3y41beVBzw/edit?usp=sharing)
* [staff idr](https://github.hetzner.co.za/hetznerZA/idr_staff/blob/master/idr_staff/lib/idr_staff/staff_idr.rb)
